// d3-based simple timline UI

var MS_IN_HOUR = 1000 * 1000 * 60 * 60;

function getSize (container) {
  var width  = container.offsetWidth - 20;
  var height = container.offsetHeight;
  return { width: width, height: height };
}

var TimeFilter = function TimeFilter(container, update) {
  var self = this;

  var id = 'time-filter';
  this._container = document.getElementById(id) || document.createElement('div');
  this._container.id = id;

  this._update = update;

  window.addEventListener('resize', function () { return self._updateSize(); });
  container.appendChild(this._container);
};


TimeFilter.prototype._updateSize = function _updateSize () {
  var ref = getSize(this._container);
    var width = ref.width;
    var height = ref.height;
  d3.select('svg').attr('width', width).attr('height', height);
};


TimeFilter.prototype.init = function init () {
  var self = this;
  this._container.innerHTML = "<svg></svg>";
  var ref = getSize(this._container);
    var width = ref.width;
    var height = ref.height;
  var buckets = this._buckets;

  var svg = d3.select('svg');
  var margin = { top: 0, right: 0, bottom: 0, left: 0 };
  var g = svg.append('g')
             .attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');
  svg
    .attr('width', width)
    .attr('height', height + 30);

  var min = buckets[0].start;
  var max = buckets[buckets.length - 1].end;

  var minValue = 0;
  var maxValue = buckets.reduce(function (a, b) { return Math.max(a, b.value); }, 0);

  var x = d3.scaleLinear()
    .domain([min, max])
    .range([0, width]);

  var y = d3.scaleLinear()
      .domain([0, maxValue])
      .range([height, 0]);

  var brush = d3.brushX()
      .extent([[0, 0], [width, height]])
      .on('start brush', brushed);

  var barWidth = width / buckets.length;

  var bar = g.selectAll('g')
      .data(buckets)
        .enter().append('g')
        .attr('transform', function (d, i) {
          return 'translate(' + i * barWidth + ',0)';
        });

  bar.append('rect')
    .attr('y', function (d) { return y(d.value); })
    .attr('height', function (d) { return height - y(d.value); })
    .attr('width', barWidth)
    .attr('class', 'bar');

  g.append('g')
      .call(brush)
      .call(brush.move, [min + MS_IN_HOUR, max - MS_IN_HOUR].map(x))
    .selectAll('.overlay')
      .on('mousedown touchstart', beforebrushed, true);

  g.append('g')
    .attr('transform', 'translate(0,' + height + ')')
    .call(d3.axisBottom(x)
      .tickFormat(function (d) {
        var date = new Date(d)
        return ((date.getHours()) + ":" + (date.getMinutes()));
      }));

  function beforebrushed() {
    d3.event.stopImmediatePropagation();
    d3.select(this.parentNode).transition().call(brush.move, x.range());
  }

  brushed([buckets[0].end, buckets[buckets.length - 1].start]);

    
  function brushed(extent) {
    extent =extent || d3.event.selection.map(x.invert, x);
    var ids = [];
    var start = Infinity, end = -Infinity;
    bar.classed('selected', function (d) {
      var inside = extent[0] <= d.start && d.end <= extent[1];
      if (inside) {
        start = Math.min(start, d.start);
        end = Math.max(end, d.end);
      }
      return inside;
    });
    appState.timeFilter = [start, end];
    self._update();
  }
};


TimeFilter.prototype.update = function update () {
  var edges = ogma.getEdges('raw')
    .filter(function (edge) { return edge.getData('type') === CONNECTS_TO; })
    .getData()
    .sort(function (e1, e2) { return e1.date - e2.date; });

  var start = edges[0].date;
  var next = start + MS_IN_HOUR;
  var value = 0;

  var buckets = [{ 
    start: edges[0].date - MS_IN_HOUR, 
    end: edges[0].date, 
    next: next, value: value
  }];
  edges.forEach(function (e, i) {
    while (e.date > next) {
      // store
      buckets.push({ start: start, end: next, value: value });
      // reset
      start = next;
      next= start + MS_IN_HOUR;
      value = 0;
    }
    value += e.query_bytes || 0;
  });
  if (value !== 0) {
    buckets.push({ start: start, end: next, value: value });
  }

  buckets.push({ start: next, end: next + MS_IN_HOUR, next: 0, value: 0 });

  this._buckets = buckets;
  this.init();
};
